// This file comes with NO WARRANY and NO SUPPORT. You use it
// at your own risk.
// This is a Scilab files used to compute the examples of the paper
// "Une mthode d'identification
//  pour un systme linaire  retards"
// "A method of identification for a linear delays system".
// AUTHORS:
// Franois Ollivier
// ALIEN, INRIA-Futurs & LIX, E'cole Polytechnique
// F-91128 Palaiseau CEDEX FRANCE
// francois.ollivier@lix.polytechnique.fr
// Sad Moutaouakil
// Brahim Sadik
// Dpartement de Mathmatiques 
// Facult des Sciences Semlalia B.P. 2390,
// Avenue Safi, Marrakech, Maroc
// s.moutaouakil@ucam.ac.ma   sadik@ucam.ac.ma
////////////////////////////////////////////////////////////////////
// A detailed version of the paper may be found at url
// http://www.lix.polytechnique.fr/~ollivier/indexEngl.html
////////////////////////////////////////////////////////////////////
// This file may be used with scilab 2.0 and 3.0
// Computation times are much longer with scilab 4.0 and the results sometimes strange...
// Scilab is available at url http://www.scilab.org/
////////////////////////////////////////////////////////////////////
// This file corresponds to Example 1 Fig. 1
// Relax. Computation takes more than 30 min on my computer...
////////////////////////////////////////////////////////////////////

//To evaluate the computation time
//Dure de l'excution
dat=getdate();
date1=(dat(7)*60+dat(8))*60+dat(9);


//Identification of the parameters of a linear system


//System order
n=2;


//Number of integrals \int t^i*exp(-lambda*t) x(T-t) dt to be used.
//Minimum is n+1.
p=(n+1)+2;

//Maximum value of degree i in \int t^i*exp(-lambda*t) x(T-t) dt
m=n+p;

//Begining and end of simulation
TFIN=60.;
TDEBUT=0.;

//Frequency of the output y
pas=0.01;
Freq=1/pas;

//Constants \lambda and \Lambda
lambda=1;
lambdaL=0.001;


//We chose the coefficients so that the system is stable.
L2=-[0.5,0.7];
aa2=-coeff(poly(L2,"s"));
bb2=2;

//Initial condition for x
x0=[20.;0.3];

//The coefficient a1 will slowly vary for t>tdisc
tdisc=30;
//We can take for AA(t) any function. (not moving too fast !)
function aa=AA(t) 
  if t<tdisc then
    aa=aa2(1:n);
             else
    aa=-[0.35, 1.2+0.02*(t-tdisc)];
  end;
endfunction 

//We could also choose a varying b if we like.
function bb=BB(t)
  bb=bb2
endfunction

//The control function.
function u=U(t)
 u=60.0*cos(1.23*t+1.3*sin(t)-0.7*cos(0.5*t))
endfunction

//Choice of the noise
//A gaussian noise
amplitudeG=5.0;
pasG=pas;
rand('normal');
for i=1:(floor(TFIN/pasG)+1)
  RandNums(i)=rand();
end;
//We may had a high frequency cos function if we like
amplitudeS=0.;
frequenceB=500;
function noise=NOISE(t)
  noise=amplitudeG*RandNums(floor(t/pasG)+1)+amplitudeS*cos(frequenceB*t)
endfunction

//Matrix allowing to express the integral of x^(i)
M0=zeros(n+1,m);
M0(1,1)=1;
MM=M0;
//Attention i=1 corresponds to integral with     exp(-lambda t)
//          i=2                                  t exp(-lambda t)
//          i=3                                  t^2 exp(-lambda t)
for i=2:m
  M1=[zeros(1,i-1),1,zeros(1,m-i)];
  for j=2:n+1
    M1=[M1;-lambda*M1(j-1,:)+(i-1)*M0(j-1,:)];
  end;
  M0=M1;
  MM=[MM;M1];
end;

//Functionnality not used for the moment !
function poids=POIDS(M)
  [d1,d2]=size(M);
  for i=1:d1 do
    for j=1:d2 do
      poids(i,j)=1;
    end;
  end;
endfunction;

//We build a differential system to do everything :
//To compute x ;
//To compute the integrals I_{f_{j},x}
//XIxIuAB=[x1,...,xn,Ix0,...,Ixn+p,Iu0,...,Iun+p,a0,...,an-1, b]
//          1      n   n+1     n+m   n+m+1 n+2m, n+2m+1 2n+2m 2n+2m+1
//And to solve the linear system by a gradient method.
function dxdixdiudab=DXDIDAB(t,XIxIuAB)
  xx=XIxIuAB(1:n);
  aa=AA(t);
  bb=BB(t);
  dx=xx(2:n);
  ab=XIxIuAB(n+2*m+1:2*n+2*m+1);
// The linear system of order n
// dx(i) = x(i+1) si i<n
// dx(n) a0*x0 + ... + an-1*xn-1
//Attention a0 correspond  aa(1) !
  dx=[dx; aa*xx(:)+bb*U(t)];
//The system whose solutions tend to 
//\int_{0}^{t}\tau^i*exp(-lambda*\tau)x(T-\tau) d\tau.
  dix=[xx(1)+cur_noise; [1:m-1]'.*XIxIuAB((n+1):(n+m-1))]-lambda*XIxIuAB((n+1):(n+m));
  diu=[U(t); [1:m-1]'.*XIxIuAB((n+m+1):(n+2*m-1))]-lambda*XIxIuAB((n+m+1):(n+2*m));
  C=[];
  for i=n+1:m
    C=[C,POIDS(i)*MM((i-1)*(n+1)+1:i*(n+1),1:m)*XIxIuAB((n+1):(n+m),1)];
  end;
  CC=C(1:n,1:m-n);CC=CC';
  CC=[CC,XIxIuAB(2*n+m+1:n+2*m,1)];
  DD=C(n+1,:)';
  dab=-lambdaL*CC'*(CC*ab-DD);
  dxdixdiudab=[dx;dix;diu;dab];
endfunction;



//In order to avoid trouble with the numerical integrator.
//It does not like the discontinuities of the noise function.
lastresu=[x0;zeros(2*m+n+1,1)];
resu=lastresu;
for t0=TDEBUT+pas:pas:TFIN do
  cur_noise=NOISE(t0);
  lastresu=ode(lastresu,t0-pas,[t0],DXDIDAB);
  resu=[resu lastresu];
end;


xset("window",1)
xbasc()
xname("X")
plot2d([TDEBUT:pas:TFIN]', [resu(1,:)'+NOISE([TDEBUT:pas:TFIN]') resu(1,:)'],..
  [1,4],"111","Noisy output@real value of x",[0,-210,TFIN,210])

a1reel=[];
for t=TDEBUT:pas:TFIN do
 aaa=AA(t);
 a1reel=[a1reel aaa(2)];
end;
xset("window",2)
xbasc()
xname("Estimations of coefficients")
plot2d([TDEBUT:pas:TFIN]',[a1reel' resu(n+2*m+1,:)' resu(n+2*m+2,:)' resu(n+2*m+3,:)'],..
  [4,1,2,3],"111","Real a1@Estimated a0@Estimated a1@Estimated b",[0,-3,TFIN,3])

//Transposition of a Maple computation (retarddiff2)..
xEvalue=-((-resu(n+2*m+2,:)-2*lambda).*resu(n+1,:)+..
          (-resu(n+2*m+1,:)+resu(n+2*m+2,:)*lambda+lambda^2).*resu(n+2,:)..
           -resu(2*n+2*m+1,:).*resu(n+m+2,:));

dxEvalue= -((-resu(n+2*m+1,:)+resu(n+2*m+2,:)*lambda+lambda^2).*resu(n+1,:)..
                    +(-lambda-resu(n+2*m+2,:)).*xEvalue-resu(2*n+2*m+1,:).*resu(n+m+1,:));


xset("window",3)
xbasc()
xname("Estimation of x")
plot2d([TDEBUT:pas:TFIN]',[xEvalue' resu(1,:)'],..
  [1,2],"111","Estimation of x@Real value of x",[0,-230,TFIN,230])
xset("window",4)
xbasc()
xname("Estimation the derivative of x")
plot2d([TDEBUT:pas:TFIN]',[dxEvalue' resu(2,:)'],..
  [1,2],"111","Estimated dx@Real dx",[0,-130,TFIN,130])

dat=getdate();

//Dure de l'excution. Execution time
(dat(7)*60+dat(8))*60+dat(9) - date1


//resu(:,3001)
//! - 179.45361 !
//! - 17.130897 !
//! - 136.82152 !
//! - 98.144266 !
//! - 147.28062 !
//! - 351.68037 !
//! - 1079.7531 !
//! - 3328.3738 !
//! - 4457.6508 !
//! - 36.739362 !
//! - 42.523067 !
//! - 69.010025 !
//! - 143.96921 !
//! - 401.54968 !
//! - 1602.4875 !
//! - 8503.1152 !
//! - 0.3503554 !
//! - 1.1940071 !
//!   1.9985224 !
//
//xEvalue(3001)
// - 179.91579      
//
//dxEvalue(3001)
// - 17.127684
//
//resu(:,6001)
//! - 98.331874 !
//! - 6.2969466 !
//! - 91.966483 !
//! - 85.52617  !
//! - 159.77438 !
//! - 435.46025 !
//! - 1442.9471 !
//! - 5042.9468 !
//! - 14840.564 !
//! - 21.625401 !
//! - 20.517753 !
//! - 38.811403 !
//! - 108.29336 !
//! - 439.54006 !
//! - 2346.1403 !
//! - 14181.284 !
//! - 0.3448155 !
//! - 1.6109682 !
//!   1.9439225 !
//
//xEvalue(6001)
//  - 98.425828    
//dxEvalue(6001)
// - 6.3801775 
//
//AA(60)
//! - 0.35  - 1.8 !  