// This file comes with NO WARRANY and NO SUPPORT. You use it
// at your own risk.
// This is a Scilab files used to compute the examples of the paper
// "Une mthode d'identification
//  pour un systme linaire  retards"
// "A method of identification for a linear delays system".
// AUTHORS:
// Franois Ollivier
// ALIEN, INRIA-Futurs & LIX, E'cole Polytechnique
// F-91128 Palaiseau CEDEX FRANCE
// francois.ollivier@lix.polytechnique.fr
// Sad Moutaouakil
// Brahim Sadik
// Dpartement de Mathmatiques 
// Facult des Sciences Semlalia B.P. 2390,
// Avenue Safi, Marrakech, Maroc
// s.moutaouakil@ucam.ac.ma   sadik@ucam.ac.ma
////////////////////////////////////////////////////////////////////
// A detailed version of the paper may be found at url
// http://www.lix.polytechnique.fr/~ollivier/indexEngl.html
////////////////////////////////////////////////////////////////////
// This file may be used with scilab 2.0 and 3.0
// Computation times are much longer with scilab 4.0 and the results sometimes strange...
// Scilab is available at url http://www.scilab.org/
////////////////////////////////////////////////////////////////////
// This file corresponds to Example 2.
// It shows how the nonlinear least square method fails when initialized
// with data too far from the solution.
////////////////////////////////////////////////////////////////////

//The inefficient method requires memory!
stacksize(70000000);

//Order of the system
n=2;

//Dbut et Fin de la simulation
TFIN=115;
TDEBUT=0.;

//The delay
R=4.;

//Measurement frequency
Freq=500;
//Saving step
pas=1/Freq;

//We chose the coefficients so that the system is stable.
L=-[0.5,0.7];
aa1=-coeff(poly(L,"s"));
bb1=2;

//If one wants to test it with varying coefficients.
//(Constants are never so constant).
function aa=AA(t) 
  aa=aa1(1,1:n);
endfunction 

function bb=BB(t)
  bb=bb1;
endfunction

//The control
function u=U(t)
 u=60.0*cos(1.23*t+.33*sin(t)-0.47*cos(0.5*t))
endfunction

//A gaussian noise
rand('normal');
noise=zeros(1,floor(TFIN/pas)+1);
for i=1:(floor(TFIN/pas)+1)
  noise(1,i)=rand();
end;


//Building the differential system
//X=[x1,...,xn]
//   1      n
function dxddx=DX(t,XX)
  aa=AA(t);
  bb=BB(t);
  dx=XX(2:n,1);
//Le systme linaire d'ordre n
// dx(i) = x(i+1) si i<n
// dx(n) a0*x0 + ... + an-1*xn-1
//Attention a0 correspond  aa(1) !
  dxddx=[dx; aa*XX(1:2,1)+bb*U(t-R)];
endfunction;

//Initial condition for x
x0=[20.;3.5];

//The solution
resu1=ode([x0],0.,[TDEBUT:pas:TFIN],DX);

//Noise level is set to zero to make things easier!
amplitudeBruit=0.0;

//We add noise to the output
Output=resu1(1,:)+amplitudeBruit*noise;
Control= U([TDEBUT:pas:TFIN]);  


xset("window",1)
xbasc()
xname("Sortie")
plot2d([TDEBUT:pas:TFIN],Output);

//A new delay
RRR=4.;

//Trying to identify parameters
//The parameters are taken as constants functions
//solution of a new differential system
//Just to pass the value without too much trouble
//This is not the most efficient method
//and not the easiest to implement!
function dxddx=DX2(t,XX2)
    aa2=[XX2(1,1),XX2(2,1)];bb2=XX2(3,1);
    dxddx=[0;0;0;XX2(5,1); aa2*XX2(4:5,1)+bb2*U(t-RRR)];
endfunction;

function jdxddx=JDX2(t,XX2)
    aa2=[XX2(1,1),XX2(2,1)];bb2=XX2(3,1);
    jdxddx=[0;0;0;XX2(5,1); aa2*XX2(4:5,1)+bb2*U(t-RRR);
                 XX2(7,1); aa2*XX2(6:7,1)+XX2(4,1);
                 XX2(9,1); aa2*XX2(8:9,1)+XX2(5,1);
                 XX2(11,1); aa2*XX2(10:11,1)+U(t-RRR);
                 XX2(13,1); aa2*XX2(12:13,1);
                 XX2(15,1); aa2*XX2(14:15,1)];
endfunction;

function tent=tentative(coe,m)
  resu2=ode([coe],0.,[TDEBUT:pas:TFIN],DX2);
  tent=(resu2(4,:)-Output)';
endfunction;

function jtent=jtentative(coe,m)
  jresu2=ode([coe;0;0;0;0;0;0;1;0;0;1],0.,[TDEBUT:pas:TFIN],JDX2);
  jtent=[jresu2(6,:);jresu2(8,:);jresu2(10,:);jresu2(12,:);jresu2(14,:)]';
endfunction;


dat=getdate();
date1=(dat(7)*60+dat(8))*60+dat(9)+dat(10)/1000;

[z1, z2] =size(Output);
[xx, vv, info] = lsqrsolve([0.0;0.0;0.0;0.0;0.0],tentative,z2,jtentative);
xx
norm(vv)
info
getdate;

//Dure de l'excution
(ans(7)*60+ans(8))*60+ans(9)+ans(10)/1000 - date1

/////////////////////////////////////////////////////////////////////////////////
// When initialized with data close to the solution, the method succeeds.
/////////////////////////////////////////////////////////////////////////////////
// lsqrsolve([-0.3;-1.;BB(0);Output(1,1);3.5],tentative,z2,jtentative);
// Dure excution      6.679
//! - 0.3500000 !
//! - 1.2       !
//!   2.        !
//!   20.000006 !
//!   3.4999894 !
//////////////////////////////////////////////////////////////////////////////////
// But initialized with zeros it fails.
//////////////////////////////////////////////////////////////////////////////////
// lsqrsolve([0.0;0.0;0.0;0.0;0.0],tentative,z2,jtentative);
// Dure excution    268.846
//!   0.0085066 !
//!   0.0330375 !
//! - 0.0598703 !
//! - 56.420063 !
//! - 1.0578044 !
//////////////////////////////////////////////////////////////////////////////////