// This file comes with NO WARRANY and NO SUPPORT. You use it
// at your own risk.
// This is a Scilab files used to compute the examples of the paper
// "Une mthode d'identification
//  pour un systme linaire  retards"
// "A method of identification for a linear delays system".
// AUTHORS:
// Franois Ollivier
// ALIEN, INRIA-Futurs & LIX, E'cole Polytechnique
// F-91128 Palaiseau CEDEX FRANCE
// francois.ollivier@lix.polytechnique.fr
// Sad Moutaouakil
// Brahim Sadik
// Dpartement de Mathmatiques 
// Facult des Sciences Semlalia B.P. 2390,
// Avenue Safi, Marrakech, Maroc
// s.moutaouakil@ucam.ac.ma   sadik@ucam.ac.ma
////////////////////////////////////////////////////////////////////
// A detailed version of the paper may be found at url
// http://www.lix.polytechnique.fr/~ollivier/indexEngl.html
////////////////////////////////////////////////////////////////////
// This file may be used with scilab 2.0 and 3.0
// Computation times are much longer with scilab 4.0 and the results sometimes strange...
// Scilab is available at url http://www.scilab.org/
////////////////////////////////////////////////////////////////////
// This file corresponds to Example 2 Table 1
////////////////////////////////////////////////////////////////////

//The non linear least square method requires memory!
stacksize(70000000);

//Order of the system
n=2;

//Beginning and end of simulation
TFIN=115;
TDEBUT=0.;

//The delay
R=4.;

//Measurement frequency
Freq=500;
//Saving step
pas=1/Freq;

//We chose the coefficients so that the system is stable.
L=-[0.5,0.7];
aa=-coeff(poly(L,"s"));aa=[aa(1,1), aa(1,2)];
bb=2;

//The control
function u=U(t)
 u=60.0*cos(1.23*t+.33*sin(t)-0.47*cos(0.5*t))
endfunction

//The derivative of the control
function du=DU(t)
 du=-60.0*(1.23+.33*cos(t)+0.47*0.5*sin(0.5*t))*sin(1.23*t+.33*sin(t)-0.47*cos(0.5*t))
endfunction

Control= U([TDEBUT:pas:TFIN]);  

//Building the differential system
//X=[x1,...,xn]
//   1      n
function dxddx=DX(t,XX)
  aa=[XX(3,1),XX(4,1)];b=XX(5,1);
  dx=XX(2,1);
//Le systme linaire d'ordre n
// dx(i) = x(i+1) si i<n
// dx(n) a0*x0 + ... + an-1*xn-1
//Attention a0 correspond  aa(1) !
  dxddx=[dx; aa*XX(1:2,1)+bb*U(t-R);0;0;0];
endfunction;

//Variant used to compute initial conditions
function dxddx=DX0(t,XX)
  aa=[XX(3,1),XX(4,1)];
  dx=XX(2,1);
//Le systme linaire d'ordre n
// dx(i) = x(i+1) si i<n
// dx(n) a0*x0 + ... + an-1*xn-1
//Attention a0 correspond  aa(1) !
  dxddx=[dx; aa*XX(1:2,1);0;0];
endfunction;


//Initial condition for x
x0=[20.;3.5];

//The solution
resu1=ode([x0;aa';bb],0.,[TDEBUT:pas:TFIN],DX);



//Trying to identify parameters
//The parameters are taken as constants functions
//solution of a new differential system
//Just to pass the value without too much trouble
function dxddx=DX2(t,XX2)
    aa2=[XX2(1,1),XX2(2,1)];bb2=XX2(3,1);RRR=XX2(6,1);
    dxddx=[0;0;0;XX2(5,1); aa2*XX2(4:5,1)+bb2*U(t-RRR);0];
endfunction;

//This more complicated version is used to compute the
//jacobian matrix of the output with respect to parameters.
//(Used by the nonlinear least quare method for better efficiency.)
function jdxddx=JDX2(t,XX2)
    aa2=[XX2(1,1),XX2(2,1)];bb2=XX2(3,1);RRR=XX2(18,1);
    jdxddx=[0;0;0;XX2(5,1); aa2*XX2(4:5,1)+bb2*U(t-RRR);
                 XX2(7,1); aa2*XX2(6:7,1)+XX2(4,1);
                 XX2(9,1); aa2*XX2(8:9,1)+XX2(5,1);
                 XX2(11,1); aa2*XX2(10:11,1)+U(t-RRR);
                 XX2(13,1); aa2*XX2(12:13,1);
                 XX2(15,1); aa2*XX2(14:15,1)
                 XX2(17,1); aa2*XX2(16:17,1)-bb2*DU(t-RRR);
                 0];
endfunction;

//The function being appoximated to 0 by the
//nonlinear least square method
function tent=tentative(coe,m)
  resu2=ode([coe],0.,[TDEBUT:pas:TFIN],DX2);
  tent=(resu2(4,:)-Output)';
endfunction;

//The jacobian of this function
function jtent=jtentative(coe,m)
  jresu2=ode([coe(1:5,1);0;0;0;0;0;0;1;0;0;1;0;0;coe(6,1)],0.,[TDEBUT:pas:TFIN],JDX2);
  jtent=[jresu2(6,:);jresu2(8,:);jresu2(10,:);jresu2(12,:);jresu2(14,:);jresu2(16,:)]';
endfunction;

//We use our method to evaluate the coefficients
exec('example2A.sci');

//To evaluate the computation time
dat=getdate();
date1=(dat(7)*60+dat(8))*60+dat(9)+dat(10)/1000;



//Used to store the results
RESU1=[];RESU2=[];

//We will evaluate the mean and standard deviations of the results
//on many computations.
for J=1:100 do

J

//A gaussian noise is computed for every test
rand('normal');
noise=zeros(1,floor(TFIN/pas)+1);
for i=1:(floor(TFIN/pas)+1)
  noise(1,i)=rand();
end;

//Noise level
amplitudeBruit=5.0;

//We add noise to the output
Output=resu1(1,:)+amplitudeBruit*noise;


//xset("window",1)
//xbasc()
//xname("Sortie")
//plot2d([TDEBUT:pas:TFIN],Output);

//Using our method
ident1=delayIdentificationOrder2(TDEBUT,TFIN,pas,Output,Control,10.,115.,15.,9,0.,50);


aa=[ident1(1,2),ident1(1,1)];
bb=ident1(1,3);


//We still need to compute the initial conditions
//In a straightforward way
resu0=ode([0;0;aa';bb],0.,[TDEBUT:pas:TFIN],DX);
resu01=ode([1;0;aa'],0.,[TDEBUT:pas:TFIN],DX0);
resu02=ode([0;1;aa'],0.,[TDEBUT:pas:TFIN],DX0);
XX0=lsq([resu01(1,:);resu02(1,:)]',(Output-resu0(1,:))');

//Storing the result
RESU1=[RESU1;[ident1(1,2),ident1(1,1),ident1(1,3),XX0',ident1(1,4)]];

[z1, z2] = size(Output);

//Using the nonlinear least square method of scilab,
//initialised with our results.
[xx, vv, info] = lsqrsolve([ident1(1,2);ident1(1,1);ident1(1,3);XX0;ident1(1,4)],tentative,z2,jtentative);
xx;
norm(vv);
info;

RESU2=[RESU2;xx'];

end;

dat=getdate();

//Dure de l'excution
(dat(7)*60+dat(8))*60+dat(9)+dat(10)/1000 - date1

mean(RESU1,'r')
variance(RESU1,'r').^(0.5)

mean(RESU2,'r')
variance(RESU2,'r').^(0.5)

///////////////////////////////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 5.0
// dure excution: 1033.176
// Sans moindres carrs :
// moyennes des paramtres: ! - 0.3497663  - 1.1980168    1.9974074    19.758674    3.8787569    3.9919    !
// carts types:            !   0.0349539    0.1032953    0.2128091    0.8226991    5.8278375    0.0664113 !
// Avec moindres carrs :
// moyennes des paramtres: ! - 0.3500386  - 1.2001188    2.0001965    20.014729    3.495359     4.0000702 !
// carts types:            !   0.0004876    0.0019610    0.0024687    0.3685455    0.5031185    0.0009000 !
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// amplitudeBruit: 10.0
// dure excution:     1077.499
// Sans moindres carrs :
// moyennes des paramtres: ! - 0.3286058  - 1.1323545    1.8668998    19.161778    1.8813005    3.94114   !
// carts types:            !   0.0581383    0.1686074    0.3504281    1.7347399    10.655605    0.1084619 !
// Avec moindres carrs :
// moyennes des paramtres: ! - 0.3499055  - 1.1996945    1.9996467    19.890598    3.6417474    3.999723  !
// carts types:            !   0.0009639    0.0040091    0.0051126    0.6161634    0.9118007    0.0017110 !
////////////////////////////////////////////////////////////////////////////////////////////////////////////
