// This file comes with NO WARRANY and NO SUPPORT. You use it
// at your own risk.
// This is a Scilab files used to compute the examples of the paper
// "Une mthode d'identification
//  pour un systme linaire  retards"
// "A method of identification for a linear delays system".
// AUTHORS:
// Franois Ollivier
// ALIEN, INRIA-Futurs & LIX, E'cole Polytechnique
// F-91128 Palaiseau CEDEX FRANCE
// francois.ollivier@lix.polytechnique.fr
// Sad Moutaouakil
// Brahim Sadik
// Dpartement de Mathmatiques 
// Facult des Sciences Semlalia B.P. 2390,
// Avenue Safi, Marrakech, Maroc
// s.moutaouakil@ucam.ac.ma   sadik@ucam.ac.ma
////////////////////////////////////////////////////////////////////
// A detailed version of the paper may be found at url
// http://www.lix.polytechnique.fr/~ollivier/indexEngl.html
////////////////////////////////////////////////////////////////////
// This file may be used with scilab 2.0 and 3.0
// Computation times are much longer with scilab 4.0 and the results sometimes strange...
// Scilab is available at url http://www.scilab.org/
////////////////////////////////////////////////////////////////////
// This file is used by the files example20.sci and example21.sci
////////////////////////////////////////////////////////////////////


//Order of the system
n=2;

//Dbut et Fin de la simulation
TFIN=115;
TDEBUT=0.;

//The delay
R=4.;

//Measurement frequency
Freq=500;
//Saving step
pas=1/Freq;

//We chose the coefficients so that the system is stable.
L=-[0.5,0.7];
aa1=-coeff(poly(L,"s"));
bb1=2;

//If one wants to test it with varying coefficients.
//(Constants are never so constant).
function aa=AA(t) 
  aa=aa1(1,1:n);
endfunction 

function bb=BB(t)
  bb=bb1;
endfunction

//The control
function u=U(t)
 u=60.0*cos(1.23*t+.33*sin(t)-0.47*cos(0.5*t))
endfunction

//A gaussian noise
rand('normal');
noise=zeros(1,floor(TFIN/pas)+1);
for i=1:(floor(TFIN/pas)+1)
  noise(1,i)=rand();
end;


//Building the differential system
//X=[x1,...,xn]
//   1      n
function dxddx=DX(t,XX)
  aa=AA(t);
  bb=BB(t);
  dx=XX(2:n,1);
//Le systme linaire d'ordre n
// dx(i) = x(i+1) si i<n
// dx(n) a0*x0 + ... + an-1*xn-1
//Attention a0 correspond  aa(1) !
  dxddx=[dx; aa*XX+bb*U(t-R)];
endfunction;

//Initial condition for x
x0=[20.;3.5];

resu1=ode([x0],0.,[TDEBUT:pas:TFIN],DX);

//Noise level
amplitudeBruit=5.0;

Output=resu1(1,:)+amplitudeBruit*noise;
Control= U([TDEBUT:pas:TFIN]);  

xset("window",1)
xbasc()
xname("Sortie")
plot2d([TDEBUT:pas:TFIN],Output);
